"""
Support for Wink binary sensors.

For more details about this platform, please refer to the documentation at
at https://home-assistant.io/components/binary_sensor.wink/
"""

from homeassistant.components.binary_sensor import BinarySensorDevice
from homeassistant.components.sensor.wink import WinkDevice
from homeassistant.helpers.entity import Entity

DEPENDENCIES = ['wink']

# These are the available sensors mapped to binary_sensor class
SENSOR_TYPES = {
    "opened": "opening",
    "brightness": "light",
    "vibration": "vibration",
    "loudness": "sound",
    "liquid_detected": "moisture",
    "motion": "motion",
    "presence": "occupancy",
    "co_detected": "gas",
    "smoke_detected": "smoke"
}


def setup_platform(hass, config, add_devices, discovery_info=None):
    """Setup the Wink binary sensor platform."""
    import pywink

    for sensor in pywink.get_sensors():
        if sensor.capability() in SENSOR_TYPES:
            add_devices([WinkBinarySensorDevice(sensor, hass)])

    for key in pywink.get_keys():
        add_devices([WinkBinarySensorDevice(key, hass)])

    for sensor in pywink.get_smoke_and_co_detectors():
        add_devices([WinkBinarySensorDevice(sensor, hass)])

    for hub in pywink.get_hubs():
        add_devices([WinkHub(hub, hass)])

    for remote in pywink.get_remotes():
        add_devices([WinkRemote(remote, hass)])

    for button in pywink.get_buttons():
        add_devices([WinkButton(button, hass)])

    for gang in pywink.get_gangs():
        add_devices([WinkGang(gang, hass)])


class WinkBinarySensorDevice(WinkDevice, BinarySensorDevice, Entity):
    """Representation of a Wink binary sensor."""

    def __init__(self, wink, hass):
        """Initialize the Wink binary sensor."""
        super().__init__(wink, hass)
        self._unit_of_measurement = self.wink.unit()
        self.capability = self.wink.capability()

    @property
    def is_on(self):
        """Return true if the binary sensor is on."""
        return self.wink.state()

    @property
    def sensor_class(self):
        """Return the class of this sensor, from SENSOR_CLASSES."""
        return SENSOR_TYPES.get(self.capability)


class WinkHub(WinkDevice, BinarySensorDevice, Entity):
    """Representation of a Wink Hub."""

    def __init(self, wink, hass):
        """Initialize the hub sensor."""
        WinkDevice.__init__(self, wink, hass)

    @property
    def is_on(self):
        """Return true if the binary sensor is on."""
        return self.wink.state()

    @property
    def device_state_attributes(self):
        """Return the state attributes."""
        return {
            'update needed': self.wink.update_needed(),
            'firmware version': self.wink.firmware_version()
        }


class WinkRemote(WinkDevice, BinarySensorDevice, Entity):
    """Representation of a Wink Lutron Connected bulb remote."""

    def __init(self, wink, hass):
        """Initialize the hub sensor."""
        WinkDevice.__init__(self, wink, hass)

    @property
    def is_on(self):
        """Return true if the binary sensor is on."""
        return self.wink.state()

    @property
    def device_state_attributes(self):
        """Return the state attributes."""
        return {
            'button_on_pressed': self.wink.button_on_pressed(),
            'button_off_pressed': self.wink.button_off_pressed(),
            'button_up_pressed': self.wink.button_up_pressed(),
            'button_down_pressed': self.wink.button_down_pressed()
        }


class WinkButton(WinkDevice, BinarySensorDevice, Entity):
    """Representation of a Wink Relay button."""

    def __init(self, wink, hass):
        """Initialize the hub sensor."""
        WinkDevice.__init__(self, wink, hass)

    @property
    def is_on(self):
        """Return true if the binary sensor is on."""
        return self.wink.state()

    @property
    def device_state_attributes(self):
        """Return the state attributes."""
        return {
            'pressed': self.wink.pressed(),
            'long_pressed': self.wink.long_pressed()
        }


class WinkGang(WinkDevice, BinarySensorDevice, Entity):
    """Representation of a Wink Relay gang."""

    def __init(self, wink, hass):
        """Initialize the gang sensor."""
        WinkDevice.__init__(self, wink, hass)

    @property
    def is_on(self):
        """Return true if the gang is connected."""
        return self.wink.state()



"""
Provides a binary sensor which is a collection of ffmpeg tools.

For more details about this platform, please refer to the documentation at
https://home-assistant.io/components/binary_sensor.ffmpeg/
"""
import asyncio
import logging
import os

import voluptuous as vol

import homeassistant.helpers.config_validation as cv
from homeassistant.components.binary_sensor import (
    BinarySensorDevice, PLATFORM_SCHEMA, DOMAIN)
from homeassistant.components.ffmpeg import (
    DATA_FFMPEG, CONF_INPUT, CONF_OUTPUT, CONF_EXTRA_ARGUMENTS)
from homeassistant.config import load_yaml_config_file
from homeassistant.const import (
    EVENT_HOMEASSISTANT_STOP, EVENT_HOMEASSISTANT_START, CONF_NAME,
    ATTR_ENTITY_ID)

DEPENDENCIES = ['ffmpeg']

_LOGGER = logging.getLogger(__name__)

SERVICE_START = 'ffmpeg_start'
SERVICE_STOP = 'ffmpeg_stop'
SERVICE_RESTART = 'ffmpeg_restart'

DATA_FFMPEG_DEVICE = 'ffmpeg_binary_sensor'

FFMPEG_SENSOR_NOISE = 'noise'
FFMPEG_SENSOR_MOTION = 'motion'

MAP_FFMPEG_BIN = [
    FFMPEG_SENSOR_NOISE,
    FFMPEG_SENSOR_MOTION
]

CONF_INITIAL_STATE = 'initial_state'
CONF_TOOL = 'tool'
CONF_PEAK = 'peak'
CONF_DURATION = 'duration'
CONF_RESET = 'reset'
CONF_CHANGES = 'changes'
CONF_REPEAT = 'repeat'
CONF_REPEAT_TIME = 'repeat_time'

DEFAULT_NAME = 'FFmpeg'
DEFAULT_INIT_STATE = True

PLATFORM_SCHEMA = PLATFORM_SCHEMA.extend({
    vol.Required(CONF_TOOL): vol.In(MAP_FFMPEG_BIN),
    vol.Required(CONF_INPUT): cv.string,
    vol.Optional(CONF_INITIAL_STATE, default=DEFAULT_INIT_STATE): cv.boolean,
    vol.Optional(CONF_NAME, default=DEFAULT_NAME): cv.string,
    vol.Optional(CONF_EXTRA_ARGUMENTS): cv.string,
    vol.Optional(CONF_OUTPUT): cv.string,
    vol.Optional(CONF_PEAK, default=-30): vol.Coerce(int),
    vol.Optional(CONF_DURATION, default=1):
        vol.All(vol.Coerce(int), vol.Range(min=1)),
    vol.Optional(CONF_RESET, default=10):
        vol.All(vol.Coerce(int), vol.Range(min=1)),
    vol.Optional(CONF_CHANGES, default=10):
        vol.All(vol.Coerce(float), vol.Range(min=0, max=99)),
    vol.Optional(CONF_REPEAT, default=0):
        vol.All(vol.Coerce(int), vol.Range(min=0)),
    vol.Optional(CONF_REPEAT_TIME, default=0):
        vol.All(vol.Coerce(int), vol.Range(min=0)),
})

SERVICE_FFMPEG_SCHEMA = vol.Schema({
    vol.Optional(ATTR_ENTITY_ID): cv.entity_ids,
})


def restart(hass, entity_id=None):
    """Restart a ffmpeg process on entity."""
    data = {ATTR_ENTITY_ID: entity_id} if entity_id else {}
    hass.services.call(DOMAIN, SERVICE_RESTART, data)


@asyncio.coroutine
def async_setup_platform(hass, config, async_add_devices, discovery_info=None):
    """Create the binary sensor."""
    from haffmpeg import SensorNoise, SensorMotion

    # check source
    if not hass.data[DATA_FFMPEG].async_run_test(config.get(CONF_INPUT)):
        return

    # generate sensor object
    if config.get(CONF_TOOL) == FFMPEG_SENSOR_NOISE:
        entity = FFmpegNoise(hass, SensorNoise, config)
    else:
        entity = FFmpegMotion(hass, SensorMotion, config)

    @asyncio.coroutine
    def async_shutdown(event):
        """Stop ffmpeg."""
        yield from entity.async_shutdown_ffmpeg()

    hass.bus.async_listen_once(
        EVENT_HOMEASSISTANT_STOP, async_shutdown)

    # start on startup
    if config.get(CONF_INITIAL_STATE):
        @asyncio.coroutine
        def async_start(event):
            """Start ffmpeg."""
            yield from entity.async_start_ffmpeg()
            yield from entity.async_update_ha_state()

        hass.bus.async_listen_once(
            EVENT_HOMEASSISTANT_START, async_start)

    # add to system
    yield from async_add_devices([entity])

    # exists service?
    if hass.services.has_service(DOMAIN, SERVICE_RESTART):
        hass.data[DATA_FFMPEG_DEVICE].append(entity)
        return
    hass.data[DATA_FFMPEG_DEVICE] = [entity]

    descriptions = yield from hass.loop.run_in_executor(
        None, load_yaml_config_file,
        os.path.join(os.path.dirname(__file__), 'services.yaml'))

    # register service
    @asyncio.coroutine
    def async_service_handle(service):
        """Handle service binary_sensor.ffmpeg_restart."""
        entity_ids = service.data.get('entity_id')

        if entity_ids:
            _devices = [device for device in hass.data[DATA_FFMPEG_DEVICE]
                        if device.entity_id in entity_ids]
        else:
            _devices = hass.data[DATA_FFMPEG_DEVICE]

        tasks = []
        for device in _devices:
            if service.service == SERVICE_START:
                tasks.append(device.async_start_ffmpeg())
            elif service.service == SERVICE_STOP:
                tasks.append(device.async_shutdown_ffmpeg())
            else:
                tasks.append(device.async_restart_ffmpeg())

        if tasks:
            yield from asyncio.wait(tasks, loop=hass.loop)

    hass.services.async_register(
        DOMAIN, SERVICE_START, async_service_handle,
        descriptions.get(SERVICE_START), schema=SERVICE_FFMPEG_SCHEMA)

    hass.services.async_register(
        DOMAIN, SERVICE_STOP, async_service_handle,
        descriptions.get(SERVICE_STOP), schema=SERVICE_FFMPEG_SCHEMA)

    hass.services.async_register(
        DOMAIN, SERVICE_RESTART, async_service_handle,
        descriptions.get(SERVICE_RESTART), schema=SERVICE_FFMPEG_SCHEMA)


class FFmpegBinarySensor(BinarySensorDevice):
    """A binary sensor which use ffmpeg for noise detection."""

    def __init__(self, hass, ffobj, config):
        """Constructor for binary sensor noise detection."""
        self._manager = hass.data[DATA_FFMPEG]
        self._state = False
        self._config = config
        self._name = config.get(CONF_NAME)
        self._ffmpeg = ffobj(
            self._manager.binary, hass.loop, self._async_callback)

    def _async_callback(self, state):
        """HA-FFmpeg callback for noise detection."""
        self._state = state
        self.hass.async_add_job(self.async_update_ha_state())

    def async_start_ffmpeg(self):
        """Start a FFmpeg instance.

        This method must be run in the event loop and returns a coroutine.
        """
        raise NotImplementedError()

    def async_shutdown_ffmpeg(self):
        """For STOP event to shutdown ffmpeg.

        This method must be run in the event loop and returns a coroutine.
        """
        return self._ffmpeg.close()

    @asyncio.coroutine
    def async_restart_ffmpeg(self):
        """Restart processing."""
        yield from self.async_shutdown_ffmpeg()
        yield from self.async_start_ffmpeg()

    @property
    def is_on(self):
        """True if the binary sensor is on."""
        return self._state

    @property
    def should_poll(self):
        """Return True if entity has to be polled for state."""
        return False

    @property
    def name(self):
        """Return the name of the entity."""
        return self._name

    @property
    def available(self):
        """Return True if entity is available."""
        return self._ffmpeg.is_running


class FFmpegNoise(FFmpegBinarySensor):
    """A binary sensor which use ffmpeg for noise detection."""

    def async_start_ffmpeg(self):
        """Start a FFmpeg instance.

        This method must be run in the event loop and returns a coroutine.
        """
        # init config
        self._ffmpeg.set_options(
            time_duration=self._config.get(CONF_DURATION),
            time_reset=self._config.get(CONF_RESET),
            peak=self._config.get(CONF_PEAK),
        )

        # run
        return self._ffmpeg.open_sensor(
            input_source=self._config.get(CONF_INPUT),
            output_dest=self._config.get(CONF_OUTPUT),
            extra_cmd=self._config.get(CONF_EXTRA_ARGUMENTS),
        )

    @property
    def sensor_class(self):
        """Return the class of this sensor, from SENSOR_CLASSES."""
        return "sound"


class FFmpegMotion(FFmpegBinarySensor):
    """A binary sensor which use ffmpeg for noise detection."""

    def async_start_ffmpeg(self):
        """Start a FFmpeg instance.

        This method must be run in the event loop and returns a coroutine.
        """
        # init config
        self._ffmpeg.set_options(
            time_reset=self._config.get(CONF_RESET),
            time_repeat=self._config.get(CONF_REPEAT_TIME),
            repeat=self._config.get(CONF_REPEAT),
            changes=self._config.get(CONF_CHANGES),
        )

        # run
        return self._ffmpeg.open_sensor(
            input_source=self._config.get(CONF_INPUT),
            extra_cmd=self._config.get(CONF_EXTRA_ARGUMENTS),
        )

    @property
    def sensor_class(self):
        """Return the class of this sensor, from SENSOR_CLASSES."""
        return "motion"



"""DO NOT MODIFY. Auto-generated by script/fingerprint_frontend."""

FINGERPRINTS = {
    "core.js": "769f3fdd4e04b34bd66c7415743cf7b5",
    "frontend.html": "d48d9a13f7d677e59b1d22c6db051207",
    "mdi.html": "7a0f14bbf3822449f9060b9c53bd7376",
    "micromarkdown-js.html": "93b5ec4016f0bba585521cf4d18dec1a",
    "panels/ha-panel-dev-event.html": "f19840b9a6a46f57cb064b384e1353f5",
    "panels/ha-panel-dev-info.html": "3765a371478cc66d677cf6dcc35267c6",
    "panels/ha-panel-dev-service.html": "1d223225c1c75083738033895ea3e4b5",
    "panels/ha-panel-dev-state.html": "8257d99a38358a150eafdb23fa6727e0",
    "panels/ha-panel-dev-template.html": "cbb251acabd5e7431058ed507b70522b",
    "panels/ha-panel-history.html": "9f2c72574fb6135beb1b381a4b8b7703",
    "panels/ha-panel-iframe.html": "d920f0aa3c903680f2f8795e2255daab",
    "panels/ha-panel-logbook.html": "313f2ac57aaa5ad55933c9bbf8d8a1e5",
    "panels/ha-panel-map.html": "13f120066c0b5faa2ce1db2c3f3cc486",
    "websocket_test.html": "575de64b431fe11c3785bf96d7813450"
}



"""
Component that will help set the ffmpeg component.

For more details about this component, please refer to the documentation at
https://home-assistant.io/components/ffmpeg/
"""
import asyncio
import logging

import voluptuous as vol

import homeassistant.helpers.config_validation as cv

DOMAIN = 'ffmpeg'
REQUIREMENTS = ["ha-ffmpeg==1.2"]

_LOGGER = logging.getLogger(__name__)

DATA_FFMPEG = 'ffmpeg'

CONF_INPUT = 'input'
CONF_FFMPEG_BIN = 'ffmpeg_bin'
CONF_EXTRA_ARGUMENTS = 'extra_arguments'
CONF_OUTPUT = 'output'
CONF_RUN_TEST = 'run_test'

DEFAULT_BINARY = 'ffmpeg'
DEFAULT_RUN_TEST = True

CONFIG_SCHEMA = vol.Schema({
    DOMAIN: vol.Schema({
        vol.Optional(CONF_FFMPEG_BIN, default=DEFAULT_BINARY): cv.string,
        vol.Optional(CONF_RUN_TEST, default=DEFAULT_RUN_TEST): cv.boolean,
    }),
}, extra=vol.ALLOW_EXTRA)


@asyncio.coroutine
def async_setup(hass, config):
    """Setup the FFmpeg component."""
    conf = config.get(DOMAIN, {})

    hass.data[DATA_FFMPEG] = FFmpegManager(
        hass,
        conf.get(CONF_FFMPEG_BIN, DEFAULT_BINARY),
        conf.get(CONF_RUN_TEST, DEFAULT_RUN_TEST)
    )

    return True


class FFmpegManager(object):
    """Helper for ha-ffmpeg."""

    def __init__(self, hass, ffmpeg_bin, run_test):
        """Initialize helper."""
        self.hass = hass
        self._cache = {}
        self._bin = ffmpeg_bin
        self._run_test = run_test

    @property
    def binary(self):
        """Return ffmpeg binary from config."""
        return self._bin

    @asyncio.coroutine
    def async_run_test(self, input_source):
        """Run test on this input. TRUE is deactivate or run correct.

        This method must be run in the event loop.
        """
        from haffmpeg import Test

        if self._run_test:
            # if in cache
            if input_source in self._cache:
                return self._cache[input_source]

            # run test
            ffmpeg_test = Test(self.binary, loop=self.hass.loop)
            success = yield from ffmpeg_test.run_test(input_source)
            if not success:
                _LOGGER.error("FFmpeg '%s' test fails!", input_source)
                self._cache[input_source] = False
                return False
            self._cache[input_source] = True
        return True
